<?php

namespace WpBeacon\Actions;

use RuntimeException;
use stdClass;
use WpBeacon\Data\PluginInfo;
use WpBeacon\Data\VersionInfo;

class GetPluginVersionsAction
{
    private static string $checkedAt;

    private static array $plugins;

    public static function all(): array
    {
        $pluginInfos = [];

        $updates = get_site_transient('update_plugins');

        self::$checkedAt = date(DATE_ATOM);
        self::$plugins = array_merge_recursive(get_plugins(), get_mu_plugins());

        if (empty($updates) || empty(self::$plugins)) {
            return $pluginInfos;
        }

        return self::mapPluginsOnUpdateTransients($updates);
    }

    private static function mapPluginsOnUpdateTransients(stdClass $updateTransient): array
    {
        $pluginInfos = [];

        foreach (self::$plugins as $key => $plugin) {

            $updateTransientEntry = $updateTransient->response[$key] ?? null;
            $noUpdateTransientEntry = $updateTransient->no_update[$key] ?? null;

            $currentVersion = $plugin['Version'];

            $latestVersion = $updateTransientEntry->new_version ?? null;
            if ($latestVersion === null && !empty($noUpdateTransientEntry)) {
                $latestVersion = $currentVersion;
            }

            $slug = explode('/', $key)[0] ?? null;

            if (empty($slug)) {
                throw new RuntimeException('Could not retrieve Plugin slug: ' . $key);
            }

            $pluginInfos[] = new PluginInfo(
                $plugin['Name'],
                $slug,
                $plugin['PluginURI'],
                new VersionInfo(
                    $plugin['Version'],
                    $latestVersion
                )
            );

        }

        return $pluginInfos;
    }
}
