<?php

namespace WpBeacon\Actions;

use stdClass;
use WP_Theme;
use WpBeacon\Data\PluginInfo;
use WpBeacon\Data\VersionInfo;

class GetThemeVersionsAction
{
    private static array $themes;

    public static function all(): array
    {
        $pluginInfos = [];

        $updates = get_site_transient('update_themes');

        self::$themes = wp_get_themes();

        if (empty($updates) || empty(self::$themes)) {
            return $pluginInfos;
        }

        return self::mapUpdateTransientOnPluginInfos($updates);
    }

    private static function mapUpdateTransientOnPluginInfos(stdClass $transient): array
    {
        return [
            ...self::mapPluginsOnPluginInfos($transient->response ?? []),
            ...self::mapPluginsOnPluginInfos($transient->no_update ?? []),
        ];
    }
    private static function mapPluginsOnPluginInfos(array $themes): array
    {
        if ($themes === []) {
            return $themes;
        }


        $pluginInfos = [];
        foreach ($themes as $key => $update) {
            $pluginInfos[] = self::mapUpdateThemesTransientItemOnPluginInfo($update, self::$themes[$key]);
        }

        return $pluginInfos;
    }

    private static function mapUpdateThemesTransientItemOnPluginInfo(array $update, WP_Theme $theme): PluginInfo
    {
        return new PluginInfo(
            $theme->get('Name'),
            $update['theme'],
            $theme->get('ThemeURI'),
            new VersionInfo(
                $theme->get('Version'),
                $update['new_version'],
            )
        );
    }
}
