<?php

namespace WpBeacon\Providers;

use WpBeacon\Actions\GetCoreVersionsAction;
use WpBeacon\Actions\GetPluginVersionsAction;
use WpBeacon\Actions\GetThemeVersionsAction;
use Throwable;

class ApiServiceProvider implements Provider
{
    public function register(): void
    {
        add_action('rest_api_init', function () {
            register_rest_route('beacon/v1', '/versions', [
                'methods' => 'GET',
                'callback' => function () {
                    return self::getEndpointData();
                },
                'permission_callback' => function () {
                    return current_user_can('manage_options');
                },
            ]);
        });
    }

    private static function getEndpointData(): array
    {
        $data = [
            'wordpress' => [],
            'plugins' => [],
            'themes' => [],
            'php' => PHP_VERSION,
            'db' => null,
        ];

        try {
            $data['wordpress'] = GetCoreVersionsAction::getCoreVersions();
            $data['plugins'] = GetPluginVersionsAction::all();
            $data['themes'] = GetThemeVersionsAction::all();
            $data['db'] = self::getDatabaseVersion();
        } catch (Throwable $e) {
            error_log($e->getMessage());
        }

        return $data;
    }

    private static function getDatabaseVersion(): string
    {
        global $wpdb;

        return $wpdb->db_server_info();
    }
}
